<?php

namespace Ardent\UtilBundle\EntityBuilder;

use Exception;
use Symfony\Component\Console\Question\ChoiceQuestion;
use Symfony\Component\Console\Style\SymfonyStyle;

class Builder
{
    /**
     * @var BaseElement[]
     */
    private array $elements = [];

    public function __construct(
        private readonly SymfonyStyle $io,
    )
    {
    }

    public function add(BaseElement|string $element): self
    {
        if (is_string($element)) {
            $element = new BaseElement($element);
        }
        $this->elements[] = $element;

        return $this;
    }

    public function getPropertyNames(): array
    {
        return array_map(fn(BaseElement $element) => $element->getProperty(), $this->elements);
    }

    private function setProperty(object $object, string $property, mixed $value): void
    {
        $setter = sprintf('set%s', ucfirst($property));
        if (!method_exists($object, $setter)) {
            throw new Exception(sprintf('Object does not have setter "%s"', $setter));
        }
        $object->$setter($value);
    }

    public function setAllByQuestions(object $object): void
    {
        foreach ($this->elements as $element) {
            $value = $element->transformValue($this->io->askQuestion($element->getQuestion()));
            $this->setProperty($object, $element->getProperty(), $value);
        }
    }

    public function setSingleByQuestion(object $object): void
    {
        $element = $this->io->askQuestion(new ChoiceQuestion('Select property to modify', $this->elements));
        $getter = sprintf('get%s', ucfirst($element->getProperty()));
        $this->io->writeln(sprintf('Current value: %s', $object->$getter()));

        $value = $element->transformValue($this->io->askQuestion($element->getQuestion()));
        $this->setProperty($object, $element->getProperty(), $value);
    }

    public function getAllPropertyValues(object $object): array
    {
        $properties = [];
        foreach ($this->elements as $element) {
            $getter = sprintf('get%s', ucfirst($element->getProperty()));
            $value = $object->$getter();
            if ($value instanceof \UnitEnum) {
                $value = $value->value;
            }
            $properties[$element->getProperty()] = $value;
        }
        return $properties;
    }

    public function getAllPropertiesTable(object $object): array
    {
        $data = [];
        foreach ($this->getAllPropertyValues($object) as $property => $value) {
            if ($value instanceof \UnitEnum) {
                $value = $value->value;
            }
            $data[] = [$property, $value];
        }
        return $data;
    }
}